
import numpy as np
from scipy.stats import norm


class ExpectedImprovement:
    def __init__(self, nr_weights, nr_samples=100, kappa=0.0, seed=None, lower_bound=-1.0, upper_bound=1.0):
        self.nr_weights = nr_weights
        self.nr_samples = nr_samples
        self.kappa = kappa
        self.lower_bound = lower_bound
        self.upper_bound = upper_bound
        self.seed = seed

    def __call__(self, gauss_process, x_observed, seed=None):
        # if seed is set for whole experiment
        if self.seed is not None:
            seed = self.seed

        # random generator
        rng = np.random.default_rng(seed)

        # get the best so far observed y
        mu = gauss_process.predict(x_observed)
        y_best = max(mu)

        # sample from surrogate
        x_test = rng.uniform(self.lower_bound, self.upper_bound, size=(self.nr_samples, self.nr_weights))
        mu, sigma = gauss_process.predict(x_test, return_std=True)

        # expected improvement
        z = (mu - y_best - self.kappa) / sigma
        ei = (mu - y_best - self.kappa) * norm.cdf(z) + sigma * norm.pdf(z)

        # get the best result and return it
        idx = np.argmax(ei)
        return x_test[idx, :]
