import unittest
from dual_quaternions import DualQuaternion
from src.interaction_policy_representation.interaction_policy_representation.models.dual_quaternion_splines import DualQuaternionSplines


class TestDualQuaternionSplines(unittest.TestCase):
    def setUp(self):
        # Example input vector (position + quaternion for 2 poses)
        self.input_vector = [1., 0., 0., 1., 0., 0., 0., 0., 1., 0., 0., 0., 1., 0.]
        # Expected dual quaternion for the first pose, adapted based on actual class functionality
        self.expected_first_dq = DualQuaternion.from_quat_pose_array(self.input_vector[:7])

    def test_parse_input_vector(self):
        trajectory = DualQuaternionSplines(self.input_vector)
        # Check if the first dual quaternion matches the expected one
        # This assumes the implementation details about how dual quaternions are stored
        self.assertAlmostEqual(trajectory.dual_quaternions[0].q_r.w, self.expected_first_dq.q_r.w, places=5)
        self.assertAlmostEqual(trajectory.dual_quaternions[0].q_r.x, self.expected_first_dq.q_r.x, places=5)
        # Add more assertions as needed for y, z, dual part

    def test_generate_trajectory(self):
        trajectory = DualQuaternionSplines(self.input_vector)
        interpolated_trajectory = trajectory.generate_trajectory()
        # Ensure the interpolated trajectory is not empty
        self.assertTrue(len(interpolated_trajectory) > 0)
        # Add more specific tests, e.g., comparing specific interpolated values

if __name__ == '__main__':
    unittest.main()
